using System.Collections.ObjectModel;
using Microsoft.Xna.Framework;

namespace EQIA
{
    /// <summary>
    /// This component will manage a collection of ControlSurfaces, since it will tamper with the back buffer it's been
    /// given an innate DrawOrder of 9000 so that it'll be drawn last, overlaying the rest of the window, this way the
    /// form the controls are attached to can still be the target of a present call event if the controls on it are using
    /// xna to draw as well.
    /// </summary>
    public class ControlSurfaceManager : DrawableGameComponent
    {
        private Collection<ControlSurface> myControls;
        private bool isInitialized;
        private bool isLoaded;

        /// <summary>
        /// Create a new instance of the control surface manager.
        /// </summary>
        /// <param name="game">The game this control surface manager belongs to.</param>
        public ControlSurfaceManager(Game game)
            : base(game)
        {
            DrawOrder = 9000;
            myControls = new Collection<ControlSurface>();
            isInitialized = false;
            isLoaded = false;
        }

        /// <summary>
        /// Initialize the control surface manager and any control surfaces that are stored.
        /// </summary>
        public override void Initialize()
        {
            base.Initialize();

            for (int i = 0; i < myControls.Count; i++)
                myControls[i].Initialize();

            isInitialized = true;
        }

        /// <summary>
        /// Load content for any control surfaces that are stored.
        /// </summary>
        protected override void LoadContent()
        {
            base.LoadContent();

            for (int i = 0; i < myControls.Count; i++)
                myControls[i].LoadContent();

            isLoaded = true;
        }

        /// <summary>
        /// Unload content for any control surfaces that are stored.
        /// </summary>
        protected override void UnloadContent()
        {
            base.UnloadContent();

            for (int i = 0; i < myControls.Count; i++)
                myControls[i].UnloadContent();

            isLoaded = false;
        }

        /// <summary>
        /// Perform update logic on any control surfaces that are stored.
        /// </summary>
        /// <param name="gameTime">Snapshot of the game time when update was called.</param>
        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);

            for (int i = 0; i < myControls.Count; i++)
                myControls[i].Update(gameTime);
        }

        /// <summary>
        /// Allow the control surfaces to first draw to texture if needed, then draw them to their
        /// target controls.
        /// </summary>
        /// <param name="gameTime">Snapshot of the game tiem when draw was called.</param>
        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            for (int i = 0; i < myControls.Count; i++)
                myControls[i].DrawToTexture();

            for (int i = 0; i < myControls.Count; i++)
                myControls[i].Draw(gameTime);
        }

        /// <summary>
        /// Add a control surface to the manager, if the manager has already had Initialize and LoadContent
        /// called, this will call them for the control surface.
        /// </summary>
        /// <param name="newControl">The control surface to add.</param>
        public void AddControl(ControlSurface newControl)
        {
            myControls.Add(newControl);

            if (isInitialized) newControl.Initialize();
            if (isLoaded) newControl.LoadContent();
        }

        /// <summary>
        /// Remove a control surface from the manager and call it's UnloadContent.
        /// </summary>
        /// <param name="oldControl">The control surface to remove.</param>
        public void RemoveControl(ControlSurface oldControl)
        {
            myControls.Remove(oldControl);
            oldControl.UnloadContent();
        }
    }
}
